@@ -144,7 +144,7 @@ const (
 	StopBreakpoint                     // The target process hit one or more software breakpoints
 	StopHardcodedBreakpoint            // The target process hit a hardcoded breakpoint (for example runtime.Breakpoint())
 	StopManual                         // A manual stop was requested
-	StopNextFinished                   // The next/step/stepout command terminated
+	StopNextFinished                   // The next/step/stepout/stepInstruction command terminated
 	StopCallReturned                   // An injected call completed
 	StopWatchpoint                     // The target process hit one or more watchpoints
 )
@@ -393,7 +393,7 @@ func (t *Target) createUnrecoveredPanicBreakpoint() {
 		panicpcs, err = FindFunctionLocation(t.Process, "runtime.fatalpanic", 0)
 	}
 	if err == nil {
-		bp, err := t.SetBreakpointWithID(unrecoveredPanicID, panicpcs[0])
+		bp, err := t.SetBreakpoint(unrecoveredPanicID, panicpcs[0], UserBreakpoint, nil)
 		if err == nil {
 			bp.Name = UnrecoveredPanic
 			bp.Variables = []string{"runtime.curg._panic.arg"}
@@ -405,7 +405,7 @@ func (t *Target) createUnrecoveredPanicBreakpoint() {
 func (t *Target) createFatalThrowBreakpoint() {
 	fatalpcs, err := FindFunctionLocation(t.Process, "runtime.throw", 0)
 	if err == nil {
-		bp, err := t.SetBreakpointWithID(fatalThrowID, fatalpcs[0])
+		bp, err := t.SetBreakpoint(fatalThrowID, fatalpcs[0], UserBreakpoint, nil)
 		if err == nil {
 			bp.Name = FatalThrow
 		}
@@ -464,11 +464,6 @@ func (t *Target) GetBufferedTracepoints() []*UProbeTraceResult {
 	return results
 }
 
-// SetNextBreakpointID sets the breakpoint ID of the next breakpoint
-func (t *Target) SetNextBreakpointID(id int) {
-	t.Breakpoints().breakpointIDCounter = id
-}
-
 const (
 	FakeAddressBase     = 0xbeef000000000000
 	fakeAddressUnresolv = 0xbeed000000000000 // this address never resloves to memory
